#ifndef CAS_CODEC_H
#define CAS_CODEC_H

#include <QObject>
#include <QtEndian>
#include <QFile>
#include <QThread>
#include <QList>
#include <QIODevice>

//    This file is part of CAS.

//    CAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    CAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#define CAS_TAG "CAS"
#define CAS_REV 1

QByteArray wrNum(uchar numOfBits, quint64 value, bool be);
quint64    rdNum(const QByteArray &dataIn, bool be);
quint64    rdNum(QIODevice *dev, uchar numOfBits, bool be);

class Channel : public QObject
{
    Q_OBJECT

private:

    quint64  pcmSample;
    quint64  xGap;
    uchar    channelId;
    uchar   *pcmBits;
    bool    *bigE;
    bool     ramping;
    bool     firstReadSample;

public:

    explicit Channel(uchar chId, uchar *bits, bool *be, QObject *parent = nullptr);

public slots:

    void toCas(uchar id, const QByteArray &pcm, bool forceWr);
    void fromCas(uchar id, const QByteArray &cas);

signals:

    void sampleRdy(uchar id, const QByteArray &data);
};

class CASCodec : public QObject
{
    Q_OBJECT

private:

    QList<QList<QByteArray> > buffer;
    QIODevice                *srcIO;
    QIODevice                *dstIO;
    QString                   inputFilePath;
    QString                   outPutFilePath;
    qint64                    lastBytes;
    uchar                     currentChannel;
    uchar                     casRev;
    uchar                     pcmBits;
    uint                      sampleRate;
    bool                      bigE;
    bool                      encode;

private slots:

    void dataIn();
    void finishedSample(uchar id, const QByteArray &data);

public:

    bool stopRequested;

    bool rdCasHeader(QIODevice *rdIO);
    bool wrCasHeader(QIODevice *wrIO);

    explicit CASCodec(QObject *parent = nullptr);

public slots:

    void setPCMBits(int numOfBits);
    void setEncodeMode(bool state);
    void setBigEndianMode(bool state);
    void setNumOfChannels(int ch);
    void setSampleRate(int rate);
    void exec(QIODevice *rdIO, QIODevice *wrIO);
    void stop();

signals:


    void encodeSample(uchar id, const QByteArray &pcm, bool forceWr);
    void decodeSample(uchar id, const QByteArray &cas);
    void delChannels();
    void updateNumOfChannels(int value);
    void updatePCMBits(int numOfBits);
    void updateSampleRate(int rate);
    void stopError(const QString &msg);
    void runningState(bool state);
    void codecStatus(const QString &msg);
};

#endif // CAS_CODEC_H
