#include "ui.h"

//    This file is part of CAS.

//    CAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    CAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

Ui::Ui(QWidget *parent) : QMainWindow(parent)
{
    QWidget     *mainWid    = new QWidget(this);
    QGridLayout *mainLayout = new QGridLayout(mainWid);
    QPushButton *start      = new QPushButton(tr("Start"), mainWid);
    QLabel      *status     = new QLabel(tr("Ready.\n"), mainWid);

    encode        = new QRadioButton(tr("Encode"), mainWid);
    decode        = new QRadioButton(tr("Decode"), mainWid);
    bigE          = new QRadioButton(tr("Big"), mainWid);
    littleE       = new QRadioButton(tr("Little"), mainWid);
    bits          = new QSpinBox(mainWid);
    numOfChannels = new QSpinBox(mainWid);
    sampleRate    = new QSpinBox(mainWid);
    inFile        = new QLineEdit(mainWid);
    outFile       = new QLineEdit(mainWid);
    pickInput     = new QPushButton(tr("Browse"), mainWid);
    pickOutput    = new QPushButton(tr("Browse"), mainWid);
    stop          = new QPushButton(tr("Stop"), this);
    progress      = new QProgressBar(mainWid);
    codec         = new CASCodec(this);
    progTimer     = new QTimer(this);
    inFileIO      = new QFile(this);
    outFileIO     = new QFile(this);

    // radio groups

    QButtonGroup *io     = new QButtonGroup(mainWid);
    QButtonGroup *endian = new QButtonGroup(mainWid);

    io->addButton(encode);
    io->addButton(decode);
    endian->addButton(bigE);
    endian->addButton(littleE);

    mainLayout->addWidget(encode, 0, 1, 1, 1);
    mainLayout->addWidget(decode, 0, 2, 1, 1);

    // column 0, labels:

    mainLayout->addWidget(new QLabel(tr("Input File: "), mainWid), 1, 0, 1, 1);
    mainLayout->addWidget(new QLabel(tr("Output File: "), mainWid), 2, 0, 1, 1);
    mainLayout->addWidget(new QLabel(tr("Bit Depth: "), mainWid), 3, 0, 1, 1);
    mainLayout->addWidget(new QLabel(tr("Channels: "), mainWid), 4, 0, 1, 1);
    mainLayout->addWidget(new QLabel(tr("Sample Rate: "), mainWid), 5, 0, 1, 1);
    mainLayout->addWidget(new QLabel(tr("Endianness: "), mainWid), 6, 0, 1, 1);

    // column 1, 2 values:

    mainLayout->addWidget(inFile, 1, 1, 1, 2);
    mainLayout->addWidget(outFile, 2, 1, 1, 2);
    mainLayout->addWidget(bits, 3, 1, 1, 1);
    mainLayout->addWidget(numOfChannels, 4, 1, 1, 1);
    mainLayout->addWidget(sampleRate, 5, 1, 1, 1);
    mainLayout->addWidget(littleE, 6, 1, 1, 1);
    mainLayout->addWidget(bigE, 6, 2, 1, 1);

    // column 3, modifiers:

    mainLayout->addWidget(pickInput, 1, 3, 1, 1);
    mainLayout->addWidget(pickOutput, 2, 3, 1, 1);

    status->setWordWrap(true);
    encode->setChecked(true);
    littleE->setChecked(true);
    numOfChannels->setMinimum(1);
    numOfChannels->setValue(2);
    sampleRate->setMinimum(1);
    sampleRate->setMaximum(400000000);
    sampleRate->setValue(44100);
    bits->setMinimum(8);
    bits->setMaximum(64);
    bits->setValue(16);
    progress->setMinimum(0);
    progress->setMaximum(1);
    progress->setValue(0);
    progTimer->setSingleShot(false);

    mainLayout->addWidget(status, 7, 1, 1, 3);
    mainLayout->addWidget(progress, 8, 1, 1, 3);
    mainLayout->addWidget(start, 9, 1, 1, 1);
    mainLayout->addWidget(stop, 9, 2, 1, 1);

    addLockoutWidget(encode);
    addLockoutWidget(decode);
    addLockoutWidget(numOfChannels);
    addLockoutWidget(sampleRate);
    addLockoutWidget(bigE);
    addLockoutWidget(littleE);
    addLockoutWidget(bits);
    addLockoutWidget(inFile);
    addLockoutWidget(outFile);
    addLockoutWidget(pickInput);
    addLockoutWidget(pickOutput);
    addLockoutWidget(start);
    setCentralWidget(mainWid);

    connect(start, SIGNAL(clicked()), this, SLOT(startProc()));
    connect(stop, SIGNAL(clicked()), this, SLOT(stopProc()));
    connect(this, SIGNAL(runCodec(QIODevice*,QIODevice*)), codec, SLOT(exec(QIODevice*,QIODevice*)));
    connect(stop, SIGNAL(clicked()), codec, SLOT(stop()));
    connect(pickInput, SIGNAL(clicked()), this, SLOT(inputFileDialog()));
    connect(pickOutput, SIGNAL(clicked()), this, SLOT(outputFileDialog()));
    connect(codec, SIGNAL(runningState(bool)), this, SLOT(codecRunningState(bool)));
    connect(codec, SIGNAL(stopError(QString)), this, SLOT(codecError(QString)));
    connect(bits, SIGNAL(valueChanged(int)), codec, SLOT(setPCMBits(int)));
    connect(codec, SIGNAL(updatePCMBits(int)), bits, SLOT(setValue(int)));
    connect(encode, SIGNAL(toggled(bool)), codec, SLOT(setEncodeMode(bool)));
    connect(bigE, SIGNAL(toggled(bool)), codec, SLOT(setBigEndianMode(bool)));
    connect(numOfChannels, SIGNAL(valueChanged(int)), codec, SLOT(setNumOfChannels(int)));
    connect(sampleRate, SIGNAL(valueChanged(int)), codec, SLOT(setSampleRate(int)));
    connect(codec, SIGNAL(updateSampleRate(int)), sampleRate, SLOT(setValue(int)));
    connect(progTimer, SIGNAL(timeout()), this, SLOT(updateProgress()));
    connect(codec, SIGNAL(codecStatus(QString)), status, SLOT(setText(QString)));

    emit numOfChannels->valueChanged(numOfChannels->value());
    emit sampleRate->valueChanged(sampleRate->value());
    emit bits->valueChanged(bits->value());
    emit encode->toggled(encode->isChecked());
    emit bigE->toggled(bigE->isChecked());
}

void Ui::addLockoutWidget(QWidget *wid)
{
    connect(this, SIGNAL(enableLockWidgets(bool)), wid, SLOT(setEnabled(bool)));
}

void Ui::inputFileDialog()
{
    inFile->setText(QFileDialog::getOpenFileName(this, tr("Input file"), QDir::homePath(), tr("*.*")));
}

void Ui::outputFileDialog()
{
    outFile->setText(QFileDialog::getSaveFileName(this, tr("Output file"), QDir::homePath()));
}

void Ui::codecRunningState(bool state)
{
    if (state)
    {
        progress->setMinimum(0);
        progress->setMaximum(inFileIO->size());
        progTimer->start(500);
    }
    else
    {
        progress->setValue(0);
        progTimer->stop();
        inFileIO->close();
        outFileIO->close();
    }

    emit enableLockWidgets(!state);
}

void Ui::stopProc()
{
    codec->stopRequested = true;
}

void Ui::startProc()
{
    inFileIO->setFileName(inFile->text());
    outFileIO->setFileName(outFile->text());

    if (!inFileIO->open(QFile::ReadOnly))
    {
        codecError(tr("Could not open the input file - ") + inFileIO->errorString() + ".");
    }
    else if (!outFileIO->open(QFile::WriteOnly | QFile::Truncate))
    {
        codecError(tr("Could not open the output file - ") + outFileIO->errorString() + ".");
    }
    else
    {
        if (encode->isChecked())
        {
            if (codec->wrCasHeader(outFileIO))
            {
                emit runCodec(inFileIO, outFileIO);
            }
            else
            {
                codecError(tr("Could not write a new CAS header to the output file."));
            }
        }
        else
        {
            if (codec->rdCasHeader(inFileIO))
            {
                emit runCodec(inFileIO, outFileIO);
            }
            else
            {
                codecError(tr("Could not read the input file's CAS header."));
            }
        }
    }
}

void Ui::codecError(const QString &msg)
{
    inFileIO->close();
    outFileIO->close();

    QMessageBox::critical(this, "", msg);
}

void Ui::updateProgress()
{
    progress->setValue(inFileIO->pos());

    if (inFileIO->atEnd())
    {
        emit stop->click();
    }
}
