#ifndef UI_H
#define UI_H

//    This file is part of CAS.

//    CAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    CAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include <QMainWindow>
#include <QLineEdit>
#include <QComboBox>
#include <QGridLayout>
#include <QWidget>
#include <QFile>
#include <QRadioButton>
#include <QButtonGroup>
#include <QLabel>
#include <QPushButton>
#include <QFileDialog>
#include <QSpinBox>
#include <QHBoxLayout>
#include <QMessageBox>
#include <QProgressBar>
#include <QTimer>

#include "cas_codec.h"

class Ui : public QMainWindow
{
    Q_OBJECT

private:

    QLineEdit    *inFile;
    QLineEdit    *outFile;
    QFile        *inFileIO;
    QFile        *outFileIO;
    QSpinBox     *bits;
    QSpinBox     *numOfChannels;
    QSpinBox     *sampleRate;
    QRadioButton *encode;
    QRadioButton *decode;
    QRadioButton *bigE;
    QRadioButton *littleE;
    QPushButton  *pickInput;
    QPushButton  *pickOutput;
    QPushButton  *stop;
    QProgressBar *progress;
    CASCodec     *codec;
    QTimer       *progTimer;

    void addLockoutWidget(QWidget *wid);

private slots:

    void codecError(const QString &msg);
    void codecRunningState(bool state);
    void inputFileDialog();
    void outputFileDialog();
    void stopProc();
    void startProc();
    void updateProgress();

public:

    Ui(QWidget *parent = 0);

signals:

    void enableLockWidgets(bool state);
    void runCodec(QIODevice *src, QIODevice *dst);
};

#endif // UI_H
